/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Florent BENOIT
 * --------------------------------------------------------------------------
 * $Id: WebContainer.java,v 1.1 2004/06/11 09:27:47 benoitf Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ant.jonasbase;

import org.objectweb.jonas.ant.JOnASBaseTask;

/**
 * Allow to configure the WebContainer service
 * @author Florent Benoit
 */
public class WebContainer extends Tasks {

    /**
     * Info for the logger
     */
    private static final String INFO = "[WebContainer] ";

    /**
     * Name of the implementation class for Jetty
     */
    private static final String JETTY_SERVICE = "org.objectweb.jonas.web.jetty50.JettyJWebContainerServiceImpl";

    /**
     * Name of the implementation class for Tomcat
     */
    private static final String TOMCAT_SERVICE = "org.objectweb.jonas.web.wrapper.CatalinaJWebContainerServiceWrapper";

    /**
     * Name of the implementation class for the default service (tomcat)
     */
    private static final String DEFAULT_SERVICE = TOMCAT_SERVICE;

    /**
     * Default port number
     */
    private static final String DEFAULT_PORT = "9000";

    /**
     * Default constructor
     */
    public WebContainer() {
        super();
    }

    /**
     * Set the port number for the WebContainer
     * @param portNumber the port for the web Container
     */
    public void setPort(String portNumber) {
        // For tomcat
        JReplace propertyReplace = new JReplace();
        propertyReplace.setConfigurationFile(JOnASBaseTask.TOMCAT_CONF_FILE);
        propertyReplace.setToken(DEFAULT_PORT);
        propertyReplace.setValue(portNumber);
        propertyReplace.setLogInfo(INFO + "Setting Tomcat port number to : " + portNumber);
        addTask(propertyReplace);

        // For Jetty
        propertyReplace = new JReplace();
        propertyReplace.setConfigurationFile(JOnASBaseTask.JETTY_CONF_FILE);
        propertyReplace.setToken(DEFAULT_PORT);
        propertyReplace.setValue(portNumber);
        propertyReplace.setLogInfo(INFO + "Setting Jetty port number to : " + portNumber);
        addTask(propertyReplace);
    }

    /**
     * Set the name of the web container : jetty or tomcat
     * @param containerName jetty or tomcat
     */
    public void setName(String containerName) {

        String service = null;

        if (containerName.equalsIgnoreCase("jetty")) {
            service = JETTY_SERVICE;
        } else {
            service = TOMCAT_SERVICE;
        }

        // Replace the service in jonas.properties file
        JReplace propertyReplace = new JReplace();
        propertyReplace.setConfigurationFile(JOnASBaseTask.JONAS_CONF_FILE);
        propertyReplace.setToken(DEFAULT_SERVICE);
        propertyReplace.setValue(service);
        propertyReplace.setLogInfo(INFO + "Setting service to : " + service);
        addTask(propertyReplace);
    }

}