/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id: AbsJmxServiceImpl.java,v 1.15 2004/09/24 09:42:09 danesa Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * JMX Service implementation. This singleton class must exist in each jonas
 * server that is to be administered via JMX. Its main role is to initialize the
 * service (the singleton object).
 * @author Guillaume Riviere
 * @author Michel Bruno
 * @author Adriana Danes
 */

public abstract class AbsJmxServiceImpl extends AbsServiceImpl implements JmxService {

    /**
     * Trace logger
     */
    private static Logger logger = null;

    /**
     * @return Logger logger object
     */
    protected static Logger getLogger() {
        return logger;
    }

    /**
     * JMX server (MBean server) instance
     */
    private MBeanServer jmxServer = null;

    /**
     * Init the Service. Create the MBeanServer. Configuration information is
     * passed through a Context object.
     * @param ctx context containing service initilaisation parameters
     * @exception ServiceException the service could not be initialized
     */
    public void doInit(Context ctx) throws ServiceException {
        logger = Log.getLogger(Log.JONAS_JMX_PREFIX);
        super.initLogger(Log.getLogger(Log.JONAS_MANAGEMENT_PREFIX));

        // Create the MBeanServer
        // ----------------------
        jmxServer = MBeanServerFactory.createMBeanServer(getDomainName());

        logger.log(BasicLevel.DEBUG, "JMX Service initialized");
    }

    /**
     * Start the Service Initialization of the service is already done.
     * @exception ServiceException the service could not be started
     */
    public abstract void doStart() throws ServiceException;

    /**
     * Stop this service
     * @exception ServiceException the service could not be stoped
     */
    public abstract void doStop() throws ServiceException;

    public abstract JMXServiceURL[] getConnectorServerURLs();

    /**
     * @return The local reference of the MBean server
     */
    public MBeanServer getJmxServer() {
        return this.jmxServer;
    }

    /**
     * Remove internal references to the MBeanServer.
     */
    protected void releaseJmxServer() {
        try {
            MBeanServerFactory.releaseMBeanServer(jmxServer);
        } catch (java.lang.IllegalArgumentException ex) {
            // The mbeanServer is not found.
            logger.log(BasicLevel.ERROR, "Cannot find the MBeanServer" + ex);
        }
    }

}