/**
 *
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JNDIResource.java,v 1.6 2004/11/26 07:57:23 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.naming;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.objectweb.jonas.management.j2eemanagement.J2EEManagedObject;
import org.objectweb.util.monolog.api.Logger;

/**
 * Implements a managed object which represents a JNDI resource.
 */
public class JNDIResource extends J2EEManagedObject {

    // Naming context
    private InitialContext ctx = null;

    // List of names of the objects bound in the naming context
    private ArrayList names = null;

    // Properties of the context environment
    private String providerUrl = null;
    private String initialContextFactory = null;
    private String pkgPrefixes = null;
    private String protocol;

    static private Logger logger = null;

    public JNDIResource(String pObjectName, InitialContext ctx, String protocol) throws NamingException {
        super(pObjectName);
        this.ctx = ctx;
        this.protocol = protocol;
        Hashtable myEnv = ctx.getEnvironment();
        providerUrl = (String) myEnv.get(Context.PROVIDER_URL);
        initialContextFactory = (String) myEnv.get(Context.INITIAL_CONTEXT_FACTORY);
        pkgPrefixes = (String) myEnv.get(Context.URL_PKG_PREFIXES);
    }

    public String getProviderUrl() {
        return providerUrl;
    }
    public String getInitialContextFactory() {
        return initialContextFactory;
    }
    public String getPkgPrefixes() {
        return pkgPrefixes;
    }

    public List getNames() throws NamingException {
        names = new ArrayList();
        String a_name = null;
        NamingEnumeration ne = null;
        ne = ctx.list("");
        for ( ; ne.hasMore() ;) {
            a_name = ((NameClassPair) ne.next()).getName();
            if (a_name != null)
                names.add(a_name);
        }
        return names;
    }
	/**
	 * @return Returns the protocol.
	 */
	public String getProtocol() {
		return protocol;
	}
}
