/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or any later
 * version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

package org.objectweb.jonas.server.discovery;

import javax.management.remote.JMXServiceURL;

/**
 * @author <a href="mailto:Takoua.Abdellatif@inria.fr">Takoua Abdellatif </a>
 * @version 1.0
 */
public class DiscEvent extends DiscMessage {
    public final static String RUNNING    = "running";
    public final static String STOPPING   = "stopping";
    private String             state      = RUNNING;
    private String             serverName = null;
    private String             domainName = null;
    private JMXServiceURL[]    urls       = null;

    /**
     * @param sourceAddress
     * @param sourcePort
     */
    public DiscEvent(String sourceAddress, int sourcePort) {
        super(sourceAddress, sourcePort);
    }

    /**
     * Constructor for a Discovery Event.
     *
     * @param sourceAddress
     *          the Ip address to use to receive a response.
     * @param sourcePort
     *          is the port used in the case of a point to point response.
     * @param state
     *          is the state of the server: RUNNING or STOPPING.
     * @param serverName
     *          is Jonas server name.
     * @param domainName
     *          is Jonas domain name.
     * @param connectorURLs
     *          contains the list of all connector urls registered in the mbean
     *          server.
     */
    public DiscEvent(String sourceAddress, int sourcePort, String serverName,
            String domainName, JMXServiceURL[] connectorURLs) {
        super(sourceAddress, sourcePort);
        this.serverName = serverName;
        this.domainName = domainName;
        this.urls = connectorURLs;
    }

    // Getters and setters of message fields

    /**
     * returns server name.
     *
     * @return serverName
     */
    public String getServerName() {
        return serverName;
    }

    /**
     * returns domain name.
     *
     * @return domain name.
     */
    public String getDomainName() {
        return domainName;
    }

    /**
     * sets the domain name.
     *
     * @param domainName
     */
    public void setDomainName(String domainName) {
        this.domainName = domainName;
    }

    /**
     * sets the serverName
     *
     * @param serverName
     */
    public void setServerName(String serverName) {
        this.serverName = serverName;
    }

    /**
     * @return the connector URL
     */
    public JMXServiceURL[] getConnectorURL(){
        return urls;
    }

    /**
     * @param connectorURL
     */
    public void setConnectorURL(JMXServiceURL[] connectorURLs) {
        this.urls = connectorURLs;
    }

    /**
     * @return server state.
     */
    public String getState() {
        return state;
    }

    /**
     * sets the server state : RUNNING or STOPPING.
     *
     * @param state
     */
    public void setState(String state) {
        this.state = state;
    }

    /**
     * The string version of the message
     */
    public String toString() {
        String str = super.getSourceAddress() + ":" + super.getSourcePort() + ":"
        + state + ":" + domainName;
        return str;

    }
}