/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or any later
 * version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * Initial developer(s): Takoua Abdellatif.
 * Contributor(s): _______________________.
 */

package org.objectweb.jonas.server.discovery;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.SocketException;
import java.net.UnknownHostException;

import javax.management.Notification;
import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * @author <a href="mailto:Takoua.Abdellatif@inria.fr">Takoua Abdellatif </a>
 * @version 1.0
 */
public class DiscoveryListener implements Runnable {
    private static int      RECEIVE_BUFFER_SIZE = 1024;
    private int             port;
    private InetAddress     destAddress;
    private Enroller        enroller;
    private int             ttl;
    private MulticastSocket multicastSocket;
    private boolean         notStopped          = true;
    private long            sequenceNumber      = 0;
    private static Logger   logger              = Log
    .getLogger(Log.JONAS_SERVER_PREFIX);

    public DiscoveryListener(Enroller enroller) {
        this.port = enroller.getListeningPort();
        this.enroller = enroller;
        try {
            this.destAddress = InetAddress.getByName(enroller.getListeningIp());
            this.ttl = enroller.getTimeToLive();
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }
    }

    /**
     * Handles the received message
     *
     * @param msg
     *          the notification message
     */
    public void handleReceivedMessage(DiscEvent msg) {

        logger
        .log(BasicLevel.DEBUG, "Enroller: notification messge received" + msg);

        //  create a notification message for all listeners of the event
        Notification notif = new Notification(msg.getState(), enroller,
                sequenceNumber++, System.currentTimeMillis());
        notif.setUserData(msg);
        enroller.sendNotification(notif);
    }

    public void run() {
        Object objReceived = null;
        ObjectInputStream in = null;
        DatagramPacket datagram = new DatagramPacket(new byte[RECEIVE_BUFFER_SIZE],
                RECEIVE_BUFFER_SIZE);

        //waitServerReady();
        try {
            multicastSocket = new MulticastSocket(port);
            multicastSocket.setTimeToLive(ttl);
            multicastSocket.joinGroup(destAddress);
        } catch (IOException e) {
            e.printStackTrace();
        }

        logger.log(BasicLevel.DEBUG, " Enroller: ip address is" + destAddress);
        logger.log(BasicLevel.DEBUG, " Enroller: port is" + port);
        try {
            while (notStopped) {
                objReceived = null;
                multicastSocket.receive(datagram);

                in = new ObjectInputStream(new ByteArrayInputStream(datagram.getData()));
                objReceived = in.readObject();

                if (objReceived != null) {
                    if (objReceived instanceof DiscEvent) {
                        DiscEvent msg = (DiscEvent) objReceived;
                        handleReceivedMessage(msg);
                    }
                }
            }
        }
        catch (SocketException e) {
            logger.log(BasicLevel.ERROR, "Enroller: Socket closed" + e);
            notStopped = false;
        } catch (IOException e1) {
            e1.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    public void stopListener()
    {
        notStopped = false;
    }

}