/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or any later
 * version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 */

package org.objectweb.jonas.server.discovery;

/**
 * @author Takoua Abdellatif
 */
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.MBeanServerNotification;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.service.ServiceManager;

/**
 * DiscoveryManager goal is to listen to a well known and reconfigurable
 * IpAddress and to give appropriate JMXURL to manage remotely Jonas server.
 *
 * @author <a href="mailto:Takoua.Abdellatif@inria.fr">Takoua Abdellatif </a>
 * @version 1.0
 */
public class DiscoveryManager implements
DiscoveryManagerMBean,
MBeanRegistration,
NotificationListener {

    private int           listeningPort = 9080;
    private String        listeningIp   = "224.224.224.224";
    private DiscoveryComm dc;
    private Thread        commDaemon;
    private MBeanServer   mbeanServer;
    private ObjectName    myOn;
    private int           ttl           = 1;

    /**
     *
     */
    public DiscoveryManager() {
        start();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#start()
     */

    public void start() {

        // creates an instance of the Discovery communication class
        dc = new DiscoveryComm(this);

        if (commDaemon == null)
            commDaemon = new Thread(dc, "commDaemon");
        commDaemon.start();
    }

    /**
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#getListeningPort()
     */
    public int getListeningPort() {
        return listeningPort;
    }

    /**
     * (non-Javadoc)
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#setListeningPort()
     */
    public void setListeningPort(int listeningPort) {
        this.listeningPort = listeningPort;

    }

    /**
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#getListeningIp()
     */
    public String getListeningIp() {
        return listeningIp;
    }

    /**
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#setListeningIP()
     */
    public void setListeningIP(String listeningIP) {
        this.listeningIp = listeningIP;

    }

    /**
     *
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#setTimeToLive()
     */
    public void setTimeToLive(int ttl) {
        this.ttl = ttl;
    }

    /**
     * @see org.objectweb.jonas.server.discovery.DiscoveryManagerMBean#getTimeToLive()
     */
    public int getTimeToLive() {
        return this.ttl;
    }

    /**
     * @see javax.management.MBeanRegistration#preRegister(javax.management.MBeanServer,
     *      javax.management.ObjectName)
     */
    public ObjectName preRegister(MBeanServer mbeanServer, ObjectName on) throws Exception {
        // set mbeanServer
        this.mbeanServer = mbeanServer;
        this.myOn = on;

        return myOn;
    }

    /**
     *
     * @see javax.management.MBeanRegistration#postRegister(java.lang.Boolean)
     */
    public void postRegister(Boolean arg0) {
        // checks the state of Jonas and sends appropriate information about Jonas
        // server and connector Urls

    }

    /**
     * @see javax.management.MBeanRegistration#preDeregister()
     */
    public void preDeregister() throws Exception {
        // stop the communication Daemon
        dc.stop();

    }

    /**
     * @see javax.management.MBeanRegistration#postDeregister()
     */
    public void postDeregister() {

    }

    /**
     *
     * @see javax.management.NotificationListener#handleNotification(javax.management.Notification,
     *      java.lang.Object)
     */
    public void handleNotification(Notification notification, Object handback) {
        if (notification.getType().equals(MBeanServerNotification.UNREGISTRATION_NOTIFICATION)) {
            ObjectName notificationSender = ((MBeanServerNotification) notification).getMBeanName();
            try {
                JmxService jmxService = (JmxService) ServiceManager.getInstance().getJmxService();
                String serverName = jmxService.getJonasServerName();
                String domainName = jmxService.getDomainName();
                ObjectName j2eeServerName = J2eeObjectName.J2EEServer(domainName, serverName);
                if (notificationSender.equals(j2eeServerName)) {
                    dc.stop();
                }
            } catch (Exception e) {
                // TO DO
            }
        }
    }

}