/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2003-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EditConnectorAction.java,v 1.8 2004/10/19 14:43:21 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.catalina;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.webapp.jonasadmin.Jlists;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;

/**
 * @author Michel-Ange ANTON
 * @author Adriana Danes - update to Tomact 5.0
 */
public class EditConnectorAction extends CatalinaBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Set up the object names of the MBeans we are manipulating
        ObjectName oObjectName = null;
        try {
            oObjectName = new ObjectName(p_Request.getParameter("select"));
        }
        catch (Exception e) {
            m_Errors.add("select", new ActionError("error.catalina.connector.select.bad"
                , p_Request.getParameter("select")));
            addGlobalError(e);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }

        // Fill in the form values for display and editing
        ConnectorForm oForm = new ConnectorForm();
        m_Session.setAttribute("catalinaConnectorForm", oForm);
        oForm.setAction("edit");
        oForm.setObjectName(oObjectName.toString());
        oForm.setDebugLvlVals(Jlists.getDebugLevels());
        oForm.setBooleanVals(Jlists.getBooleanValues());

        // Populate
        try {

            // General properties
            oForm.setScheme(getStringAttribute(oObjectName, "scheme"));
            String sHandlerClassName = getStringAttribute(oObjectName, "protocolHandlerClassName");
            int period = sHandlerClassName.lastIndexOf('.');
            String sHandlerType = sHandlerClassName.substring(period + 1);
            oForm.setConnectorType("HTTPS");
            if ("JkCoyoteHandler".equalsIgnoreCase(sHandlerType)) {
                oForm.setConnectorType("AJP");
            } else if (("Http11Protocol".equalsIgnoreCase(sHandlerType))
                && ("http".equalsIgnoreCase(oForm.getScheme()))) {
                oForm.setConnectorType("HTTP");
            }

            oForm.setAcceptCountText(toStringIntegerAttribute(oObjectName, "acceptCount"));
            oForm.setConnTimeOutText(toStringIntegerAttribute(oObjectName, "connectionTimeout"));
            oForm.setDebugLvl(toStringIntegerAttribute(oObjectName, "debug"));
            oForm.setBufferSizeText(toStringIntegerAttribute(oObjectName, "bufferSize"));
            oForm.setEnableLookups(getBooleanAttribute(oObjectName, "enableLookups"));
            oForm.setAddress(getStringAttribute(oObjectName, "address"));

            // Ports
            oForm.setPortText(toStringIntegerAttribute(oObjectName, "port"));
            oForm.setRedirectPortText(toStringIntegerAttribute(oObjectName, "redirectPort"));

            // Threads
            Object oAttribute = JonasManagementRepr.getAttribute(oObjectName, "minSpareThreads");
            String className = oAttribute.getClass().getName();
            if (className.equals("java.lang.String")) {
            	oForm.setMinSpareThreadsText((String) oAttribute);
            } else if (className.equals("java.lang.Integer")) {
            	oForm.setMinSpareThreadsText(((Integer) oAttribute).toString());
            }
            oAttribute = JonasManagementRepr.getAttribute(oObjectName, "maxSpareThreads");
            className = oAttribute.getClass().getName();
            if (className.equals("java.lang.String")) {
            	oForm.setMaxSpareThreadsText((String) oAttribute);
            } else if (className.equals("java.lang.Integer")) {
            	oForm.setMaxSpareThreadsText(((Integer) oAttribute).toString());
            }
            oAttribute = JonasManagementRepr.getAttribute(oObjectName, "maxThreads");
            className = oAttribute.getClass().getName();
            if (className.equals("java.lang.String")) {
            	oForm.setMaxThreadsText((String) oAttribute);
            } else if (className.equals("java.lang.Integer")) {
            	oForm.setMaxThreadsText(((Integer) oAttribute).toString());
            }
            
            // Supported by HTTP and HTTPS only
            if (!("AJP".equalsIgnoreCase(oForm.getConnectorType()))) {
                oForm.setProxyName(getStringAttribute(oObjectName, "proxyName"));
                oForm.setProxyPortText(toStringIntegerAttribute(oObjectName, "proxyPort"));
            }

            if ("HTTPS".equalsIgnoreCase(oForm.getConnectorType())) {
                // Initialize rest of variables.
                // These are set only for SSL connectors.
                oForm.setClientAuthentication(getBooleanAttribute(oObjectName, "clientAuth"));
                oForm.setKeyStoreFileName(getStringAttribute(oObjectName, "keystoreFile"));
                oForm.setKeyStorePassword(getStringAttribute(oObjectName, "keystorePass"));
            }
            // Force the node selected in tree
            String nodeName = getTreeBranchName(DEPTH_SERVER) + WhereAreYou.NODE_SEPARATOR 
				+ "protocols" + WhereAreYou.NODE_SEPARATOR
				+ "connectors" + WhereAreYou.NODE_SEPARATOR
				//+ m_WhereAreYou.getCurrentCatalinaDomainName() + WhereAreYou.NODE_SEPARATOR
				+ oForm.getPortText();
            m_WhereAreYou.selectNameNode(nodeName, true);
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the connector display page
        return (p_Mapping.findForward("Catalina Connector"));
    }
}
