/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.objectweb.jonas.webapp.jonasadmin.joramplatform;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Set;
import java.util.Vector;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;

/**
 * @author Adriana Danes
 */

public class ListLocalDestinationsAction extends EditJoramBaseAction {

    ObjectName ejbServiceObjectName = null;
    String[] asParam = null;
    String[] asSignature = null;
    
    // --------------------------------------------------------- Public Methods
    public ActionForward executeAction(ActionMapping pMapping, ActionForm pForm
        , HttpServletRequest pRequest, HttpServletResponse pResponse)
        throws IOException, ServletException {

        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(DEPTH_DOMAIN) + WhereAreYou.NODE_SEPARATOR
                + "joramplatform" + WhereAreYou.NODE_SEPARATOR
                + "joramlocalserver", true);
        
        // Get reference on EJB container service in order to determine dependencies 
        // between the deployed EJBs and JMS destinations
        initRefs();
        
        // Get all destinations
        String asDestName = null;
        try {
            // Object name used for JmslocalServer MBean
            ObjectName oObjectName = JonasObjectName.joramLocalServer();
            ArrayList alQueues = new ArrayList();
            // get queues
            Vector v = (Vector) getListAttribute(oObjectName, "LocalQueuesNames");
            if (v != null) {
                for (int i = 0; i < v.size(); i++) {
                    asDestName = (String) v.get(i);
                    boolean deps = hasDeps(asDestName);
                    alQueues.add(new DestinationItem(asDestName, "queue", deps));
                }
            }
            // get topicss
            v = (Vector) getListAttribute(oObjectName, "LocalTopicsNames");
            ArrayList alTopics = new ArrayList();
            if (v != null) {
                for (int i = 0; i < v.size(); i++) {
                    asDestName = (String) v.get(i);
                    boolean deps = hasDeps(asDestName);
                    alTopics.add(new DestinationItem(asDestName, "topic", deps));
                }
            }

            // merge alQueues and alTopics in alDestinations
            ArrayList alDestinations = new ArrayList(alQueues);
            alDestinations.addAll(alTopics);

            pRequest.setAttribute("destinationsList", alDestinations);
        } catch (Throwable t) {
            return (treatError(t, pMapping, pRequest));
        }

        // Forward to the jsp.
        return (pMapping.findForward("Joram Destinations"));
    }

    private void initRefs() {
        asParam = new String[1];
        asSignature = new String[1];
        asSignature[0] = "java.lang.String";
        ejbServiceObjectName = JonasObjectName.ejbService();
        if (!JonasManagementRepr.isRegistered(ejbServiceObjectName)) {
            ejbServiceObjectName = null;
        }
    }
    
    private boolean hasDeps(String pDestName) {
        boolean ret = false;
        if (ejbServiceObjectName != null) {
            asParam[0] = pDestName;
            Set deps = (Set) JonasManagementRepr.invoke(
                    ejbServiceObjectName, "getJmsDestinationDependence",
                    asParam, asSignature);
            if (!deps.isEmpty()) {
                ret = true;
            }
        }
        return ret;
    }
}
