/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ApplyCatalinaAccessLoggerAction.java,v 1.6 2004/10/22 13:53:33 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.logging;

import java.io.IOException;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.CatalinaObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminException;
import org.objectweb.jonas.webapp.jonasadmin.WhereAreYou;

/**
 * @author Michel-Ange ANTON
 */
public class ApplyCatalinaAccessLoggerAction extends BaseLoggerAction {

    /**
     * Signature for the <code>createStandardConnector</code> operation.
     */
    private String sa_CreateAccessLogger[] = {
        "java.lang.String"
    };

    private static String sDefaultForward = "ActionEditWebAppCatalina";

// --------------------------------------------------------- Protected Variables

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Default forward
        ActionForward oForward = null;

        // Form used
        CatalinaAccessLogValveForm oForm = (CatalinaAccessLogValveForm) p_Form;

        // Populate
        try {
            // Perform a "Create WebApp" transaction (if requested)
            if ("create".equals(oForm.getAction()) == true) {
                oForward = createAccessLogger(oForm, p_Mapping, p_Request);
            }
            // Save in memory the new datas
            else if ("edit".equals(oForm.getAction()) == true) {
                oForward = populateMbean(oForm
                    , p_Mapping.findForward("ActionEditCatalinaAccessLogger"), p_Mapping, p_Request);
            }
        }
        catch (JonasAdminException e) {
            // Build error
            m_Errors.add("logger.catalina.access", new ActionError(e.getId()));
            saveErrors(p_Request, m_Errors);
            // Return to the current page
            oForward = new ActionForward(p_Mapping.getInput());
        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            oForward = p_Mapping.findForward("Global Error");
        }

        // Next Forward
        return oForward;
    }

    /**
     * Create a new access logger.
     * @param p_Form The current form
     * @param p_Mapping The current mapping
     * @param p_Request The current request
     * @return The forward to go to the next page
     * @throws Exception
     */
    protected ActionForward createAccessLogger(CatalinaAccessLogValveForm p_Form
        , ActionMapping p_Mapping, HttpServletRequest p_Request)
        throws Exception {
        Object values[] = null;
        // Look up the Catalina MBeanFactory
        ObjectName onFactory = CatalinaObjectName.catalinaFactory();
        // Create a new Access Logger Valve object
        values = new Object[1];
        // parent
        values[0] = CatalinaObjectName.catalinaHost(m_WhereAreYou.getCurrentCatalinaDomainName()
            , m_WhereAreYou.getCurrentCatalinaDefaultHostName()).toString();
        p_Form.setObjectName((String) JonasManagementRepr.invoke(onFactory
            , "createAccessLoggerValve", values, sa_CreateAccessLogger));

        // Populate
        ActionForward oForward = populateMbean(p_Form, p_Mapping.findForward("ActionListLoggers")
            , p_Mapping, p_Request);

        // refresh tree
        refreshTree(p_Request);
        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(DEPTH_SERVER) + WhereAreYou.NODE_SEPARATOR
            + "logging" + WhereAreYou.NODE_SEPARATOR + LoggerItem.LOGGER_CATALINA_ACCESS, true);

        // Return the next forward
        return oForward;
    }

    /**
     * Populate the Mbean Access Logger.
     * @param p_Form The current form
     * @param p_Forward The current forward
     * @param p_Mapping The current mapping
     * @param p_Request The current request
     * @return If 'save' is requested then return the new forward to save
     * else return the current forward
     * @throws Exception
     */
    protected ActionForward populateMbean(CatalinaAccessLogValveForm p_Form
        , ActionForward p_Forward, ActionMapping p_Mapping, HttpServletRequest p_Request)
        throws Exception {
        ActionForward oForward = p_Forward;
        // Access logger
        ObjectName on = new ObjectName(p_Form.getObjectName());
        setStringAttribute(on, "directory", p_Form.getDirectory());
        setStringAttribute(on, "prefix", p_Form.getPrefix());
        setStringAttribute(on, "suffix", p_Form.getSuffix());
        setBooleanAttribute(on, "resolveHosts", p_Form.isResolveHosts());
        setBooleanAttribute(on, "rotatable", p_Form.isRotatable());
        setStringAttribute(on, "pattern", p_Form.getPattern());
        // Save in configuration file
        if (p_Form.isSave() == true) {
            //ObjectName onServer = CatalinaObjectName.catalinaServer();
            //JonasManagementRepr.invoke(onServer, "store", null, null);
            p_Form.setSave(false);
            p_Request.setAttribute("forward", p_Forward.getName());
            oForward = p_Mapping.findForward("ActionEditServletServer");
        }
        return oForward;
    }

}
