/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JService.java,v 1.2 2004/11/19 09:23:00 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.ws.axis;

import java.io.InputStream;
import java.net.URL;
import java.rmi.Remote;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;

import org.apache.axis.EngineConfiguration;
import org.apache.axis.client.Service;
import org.apache.axis.wsdl.gen.Parser;

/**
 * JService is the JOnAS J2EE layer on top of axis Service implementation. It is
 * currently a no-op class.
 *
 * @author Guillaume Sauthier
 */
public class JService extends Service {

    /**
     * Internal Map that store service-endpoint-interface names to wsdl:port QName
     */
    private Map classname2wsdlPort = null;

    /**
     * @see org.apache.axis.client.Service#Service()
     */
    public JService() {
        super();
    }

    /**
     * @see org.apache.axis.client.Service#Service(javax.xml.rpc.QName)
     */
    public JService(QName serviceName) {
        super(serviceName);
    }

    /**
     * @see org.apache.axis.client.Service#Service(org.apache.axis.EngineConfiguration)
     */
    public JService(EngineConfiguration config) {
        super(config);
    }

    /**
     * @see org.apache.axis.client.Service#Service(java.net.URL, javax.xml.rpc.QName)
     */
    public JService(URL wsdlDoc, QName serviceName) throws ServiceException {
        super(wsdlDoc, serviceName);
    }

    /**
     * @see org.apache.axis.client.Service#Service(org.apache.axis.wsdl.gen.Parser, javax.xml.rpc.QName)
     */
    public JService(Parser parser, QName serviceName) throws ServiceException {
        super(parser, serviceName);
    }

    /**
     * @see org.apache.axis.client.Service#Service(java.lang.String, javax.xml.rpc.QName)
     */
    public JService(String wsdlLocation, QName serviceName) throws ServiceException {
        super(wsdlLocation, serviceName);
    }

    /**
     * @see org.apache.axis.client.Service#Service(java.io.InputStream, javax.xml.rpc.QName)
     */
    public JService(InputStream wsdlInputStream, QName serviceName) throws ServiceException {
        super(wsdlInputStream, serviceName);
    }

    /**
     * @see javax.xml.rpc.Service#getPort(java.lang.Class)
     */
    public Remote getPort(Class proxyInterface) throws ServiceException {
        if (this.classname2wsdlPort != null) {
            QName portname = (QName) this.classname2wsdlPort.get(proxyInterface.getName());
            if (portname != null) {
                return getPort(portname, proxyInterface);
            } else {
                return super.getPort(proxyInterface);
            }
        }
        return super.getPort(proxyInterface);
    }

    /**
     * Assign the classname 2 port Map.
     * @param map Map to be used
     */
    public void assignSEIClassnameToWSDLPort(Map map) {
        this.classname2wsdlPort = map;
    }
}