/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JServiceProxy.java,v 1.2 2004/11/19 09:23:00 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.ws.axis;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;


/**
 * <code>JServiceProxy</code>
 *
 * @author Guillaume Sauthier
 */
public class JServiceProxy implements InvocationHandler {

    /**
     * inner JService implementation.
     * used for delegation
     */
    private JService service = null;

    /**
     * Forbidden method : Service.getTypeMappingRegistry()
     */
    private static Method getTypeMappingRegistryMethod = null;

    /**
     * Forbidden method : Service.getHandlerRegistry()
     */
    private static Method getHandlerRegistryMethod = null;

    /**
     * Constructs a new JServiceProxy wrapping given JService instance.
     * @param service the wrapped JService instance
     * @throws ServiceException should be never thrown
     */
    public JServiceProxy(JService service) throws ServiceException {
        this.service = service;
        try {
            getTypeMappingRegistryMethod = Service.class.getDeclaredMethod("getTypeMappingRegistry", new Class[]{});
            getHandlerRegistryMethod = Service.class.getDeclaredMethod("getHandlerRegistry", new Class[]{});
        } catch (Exception e) {
            throw new ServiceException(e);
        }
    }

    /**
     * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object, java.lang.reflect.Method, java.lang.Object[])
     */
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        // avoid getHandlerRegistry method call
        if (getHandlerRegistryMethod.equals(method)) {
            throw new UnsupportedOperationException("J2EE components shouldn't use getHandlerRegistry method");
        }
        // avoid getTypeMappingRegistry method call
        if (getTypeMappingRegistryMethod.equals(method)) {
            throw new UnsupportedOperationException("J2EE components shouldn't use getTypeMappingRegistry method");
        }
        //delegate
        try {
            return method.invoke(service, args);
        } catch (InvocationTargetException ite) {
            throw ite.getTargetException();
        }
    }

}
