/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JServiceEndpoint.java,v 1.3 2004/06/08 08:38:00 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.container;

import java.rmi.NoSuchObjectException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.objectweb.jonas_ejb.deployment.api.MethodDesc;
import org.objectweb.jonas_ejb.deployment.api.SessionStatelessDesc;
import org.objectweb.jonas_ejb.lib.EJBInvocation;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author Guillaume Sauthier
 */
public class JServiceEndpoint extends PortableRemoteObject implements Remote {

    /**
     * switch instance to use
     */
    protected JSessionSwitch bs;

    /**
     * deployment descriptor
     */
    protected SessionStatelessDesc dd;

    /**
     * bean factory
     */
    protected JStatelessFactory bf;

    /**
     * registered flag
     */
    protected boolean unregistered = true;

    /**
     * @throws java.rmi.RemoteException
     */
    public JServiceEndpoint(SessionStatelessDesc sd, JStatelessFactory sf) throws RemoteException {
        super();
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        dd = sd;
        bf = sf;
        _create();
    }

    // ---------------------------------------------------------------
    // other public methods
    // ---------------------------------------------------------------

    /**
     * register this bean to JNDI (rebind)
     */
    protected void register() throws NamingException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        String name = dd.getJndiServiceEndpointName();
        // register in default InitialContext
        bf.getInitialContext().rebind(name, this);
        unregistered = false;
    }

    /**
     * unregister this bean in JNDI (unbind)
     */
    protected void unregister() throws NamingException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        String name = dd.getJndiServiceEndpointName();
        // unregister in default InitialContext
        bf.getInitialContext().unbind(name);

        try {
            PortableRemoteObject.unexportObject(this);
        } catch (NoSuchObjectException e) {
            TraceEjb.interp.log(BasicLevel.ERROR, "unexportObject(JServiceEndpoint) failed: " + e);
        }
        unregistered = true;
    }

    /**
     * preInvoke is called before any request.
     * @param txa Transaction Attribute (Supports, Required, ...)
     * @return A RequestCtx object
     * @throws RemoteException preInvoke fail
     */
    public RequestCtx preInvoke(int txa) throws RemoteException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        return bf.preInvokeRemote(txa);
    }

    /**
     * Check if the access to the bean is authorized
     * @param ejbInv object containing security signature of the method, args of
     *        method, etc
     */
    public void checkSecurity(EJBInvocation ejbInv) {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        bf.checkSecurity(ejbInv);
    }

    /**
     * postInvoke is called after any request.
     * @param rctx The RequestCtx that was returned at preInvoke()
     * @throws RemoteException postInvoke fail
     */
    public void postInvoke(RequestCtx rctx) throws RemoteException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        bf.postInvokeRemote(rctx);
    }

    /**
     * create method
     */
    public void _create() throws RemoteException {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");

        RequestCtx rctx = preInvoke(MethodDesc.TX_NOT_SET);
        try {
            bs = bf.createEJB();
        } catch (RuntimeException e) {
            rctx.sysExc = e;
            throw new RemoteException("RuntimeException thrown by an enterprise Bean", e);
        } catch (Error e) {
            rctx.sysExc = e;
            throw new RemoteException("Error thrown by an enterprise Bean", e);
        } catch (RemoteException e) {
            rctx.sysExc = e;
            throw e;
        } finally {
            postInvoke(rctx);
        }
    }

}