/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JSessionHandle.java,v 1.9 2004/11/09 16:26:13 durieuxp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.io.IOException;
import java.io.Serializable;
import java.rmi.RemoteException;

import javax.ejb.EJBObject;
import javax.ejb.Handle;
import javax.ejb.spi.HandleDelegate;
import javax.naming.NamingException;

import org.objectweb.jonas.naming.NamingManager;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class implements javax.ejb.Handle interface. For a Session Bean a Handle
 * is directly its RMI Reference because its life time is limited by the JOnAS
 * Server life time. (no need to retrieve it after the JOnAS Server has been
 * restarted)
 * @author Philippe Coq
 */
public class JSessionHandle implements Handle, Serializable {

    /**
     * @serial
     */
    private EJBObject ejbO = null;

    /**
     * constructor
     * @param jb the EJBObject represented by this handle.
     */
    public JSessionHandle(EJBObject jb) {
        TraceEjb.interp.log(BasicLevel.DEBUG, "");
        ejbO = jb;
    }

    /**
     * @return the EJBObject represented by this handle.
     * @throws RemoteException: The EJB object could not be obtained because of
     *         a system-level failure.
     */
    public EJBObject getEJBObject() throws RemoteException {
        return ejbO;
    }

    /**
     * Specific implementation of serialization.
     * Must call HandleDelegate.writeEJBObject, as specified in 19.5.5.1 of spec EJB 2.1
     * @param out The output stream used to write object
     * @throws IOException error when writing object.
     */
    private void writeObject(java.io.ObjectOutputStream out)
    throws IOException {
        HandleDelegate hdld;
        try {
            hdld = JHandleDelegate.getHandleDelegate();
        } catch (NamingException e) {
            throw new IOException("Cannot get HandleDelegate");
        }
        hdld.writeEJBObject(ejbO, out);
    }

    /**
     * Specific implementation of deserialization.
     * Must call HandleDelegate.readEJBObject, as specified in 19.5.5.1 of spec EJB 2.1
     * @param in The input Stream from where is read the object.
     * @throws IOException error when reading object.
     * @throws ClassNotFoundException -
     */
    private void readObject(java.io.ObjectInputStream in)
    throws IOException, ClassNotFoundException {
        HandleDelegate hdld;
        try {
            hdld = JHandleDelegate.getHandleDelegate();
        } catch (NamingException e) {
            throw new IOException("Cannot get HandleDelegate");
        }
        ejbO = hdld.readEJBObject(in);
    }

}