/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TraceEjb.java,v 1.31 2004/08/05 09:13:59 durieuxp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

/**
 * For message logging
 * @author Sebastien Chassande-Barrioz sebastien.chassande@inrialpes.fr
 */

class DummyLogger extends LoggerImpl {

    /**
     * Dummy Logger used when jotm classes are used on client side
     */
    public void log(int level, java.lang.Object o) {
    }
}

public class TraceEjb {

    public static Logger logger = new DummyLogger();
    public static Logger query = new DummyLogger();
    public static Logger interp = new DummyLogger();
    public static Logger genclass = new DummyLogger();
    public static Logger coherence = new DummyLogger();
    public static Logger thread = new DummyLogger();
    public static Logger synchro = new DummyLogger();
    public static Logger ssfpool = new DummyLogger();
    public static Logger swapper = new DummyLogger();
    public static Logger mapper = new DummyLogger();
    public static Logger context = new DummyLogger();
    static public Logger security = new DummyLogger();
    static public Logger factory = new DummyLogger();
    public static Logger mdb = new DummyLogger();
    public static Logger tx = new DummyLogger();
    public static Logger txlistener = new DummyLogger();
    public static Logger dd = new DummyLogger();
    public static Logger loaderlog = new DummyLogger();

    public static final String prefix = "org.objectweb.jonas_ejb";

    public static LoggerFactory loggerFactory = null;

    /**
     * Configure loggers for monolog
     * @param lf the logger factory
     */
    public static void configure(LoggerFactory lf) {
        loggerFactory = lf;
        logger = lf.getLogger(prefix);
        interp = lf.getLogger(prefix + ".interp");
        query = lf.getLogger(prefix + ".query");
        genclass = lf.getLogger(prefix + ".genclass");
        coherence = lf.getLogger(prefix + ".coherence");
        thread = lf.getLogger(prefix + ".thread");
        synchro = lf.getLogger(prefix + ".synchro");
        ssfpool = lf.getLogger(prefix + ".ssfpool");
        swapper = lf.getLogger(prefix + ".swapper");
        mapper = lf.getLogger(prefix + ".mapper");
        context = lf.getLogger(prefix + ".context");
        security = lf.getLogger(prefix + ".security");
        factory = lf.getLogger(prefix + ".factory");
        mdb = lf.getLogger(prefix + ".mdb");
        tx = lf.getLogger(prefix + ".tx");
        txlistener = lf.getLogger(prefix + ".txlistener");
        dd = lf.getLogger(prefix + ".dd");
        loaderlog = lf.getLogger("org.objectweb.jonas.loader");
    }

    public static boolean isVerbose() {
        return (logger != null) && logger.isLoggable(BasicLevel.WARN);
    }

    public static boolean isDebugThread() {
        return (thread != null) && thread.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugSynchro() {
        return (synchro != null) && synchro.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugSsfpool() {
        return (ssfpool != null) && ssfpool.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugContext() {
        return (context != null) && context.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugSwapper() {
        return (swapper != null) && swapper.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugMapper() {
        return (mapper != null) && mapper.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugIc() {
        return (interp != null) && interp.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugSecurity() {
        return (security != null) && security.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugFactory() {
        return (factory != null) && factory.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugJms() {
        return (mdb != null) && mdb.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugTx() {
        return (tx != null) && tx.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugTxlistener() {
        return (txlistener != null) && txlistener.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugDd() {
        return (dd != null) && dd.isLoggable(BasicLevel.DEBUG);
    }

    public static boolean isDebugQuery() {
        return (query != null) && query.isLoggable(BasicLevel.DEBUG);
    }

}
